package com.usaepaydemo;

import android.app.Activity;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.ScrollView;
import android.widget.TextView;

import com.usaepay.middleware.interfaces.OnLog;
import com.usaepay.middleware.publicclasses.MIDDLEWARE_STATUS;
import com.usaepay.middleware.publicclasses.TRANS_STATUS;
import com.usaepay.middleware.publicclasses.TerminalConfig;
import com.usaepay.middleware.publicclasses.UEMiddleware;
import com.usaepay.middleware.publicclasses.UEMiddlewareInterface;
import com.usaepay.middleware.struct.UEMTransactionResult;
import com.usaepay.middleware.struct.UE_ERROR;

import org.json.JSONObject;

import java.util.HashMap;

@SuppressWarnings("unchecked")
public class ProcessingActivity extends AppCompatActivity {

    private final String TAG = ProcessingActivity.class.getSimpleName();

    private HashMap<String, String> transInfo;
    private TransactionHelper th;
    private Activity parent;
    private Button btn_start_trans;
    private Button btn_cancel_trans;
    private TextView tv_status;
    private TextView tv_trans_status;
    private TextView tv_proc_log;
    private ScrollView sv_log;
    private String mConnType;
    private String mLog;
    private String sourceKey;
    private String pin;
    private String host;

    @Override
    protected void onCreate(Bundle savedState) {
        super.onCreate(savedState);
        setContentView(R.layout.activity_processing);

		// Grab references to UI elements in this Activity.
        tv_status        = (TextView) findViewById(R.id.processing_status);
        tv_trans_status  = (TextView) findViewById(R.id.trans_status);
        tv_proc_log      = (TextView) findViewById(R.id.processing_log);
        btn_start_trans  = (Button) findViewById(R.id.start_trans);
        btn_cancel_trans = (Button) findViewById(R.id.cancel_trans);
        sv_log = (ScrollView) findViewById(R.id.sv_logs);

        parent = this;

        unbundleTransInfo();

        th = new TransactionHelper();
        th.startTransaction();
    }

	/**
	 * Disconnect the device and phone/tablet when 'Back' button is pressed.
	 */
    @Override
    public void onBackPressed() {
        th.disconnect();
        finish();
    }

	/**
	 * Set the Transaction Helper object to 'null' so that it gets garbage collected.
	 */
    @Override
    protected void onDestroy() {
        super.onDestroy();
        th = null;
        parent = null;
    }

    /**
     * Stores transaction info into global variables for ease of use throughout this Activity.
     */
    private void unbundleTransInfo() {
        Bundle b = getIntent().getExtras();
        transInfo = (HashMap) b.getSerializable("TransInfo");

        mConnType = transInfo.get("conn_type");

        sourceKey = b.getString("source_key");
        pin       = b.getString("pin");
        host      = b.getString("host");
    }

    /**
     * TransactionHelper SubClass
     *
     * Implements callback methods called during a transaction.
     */
    class TransactionHelper implements UEMiddlewareInterface, OnLog {

        private UEMiddleware ueMiddleware;

        /**
         * Constructor
         */
        TransactionHelper() {
        }

		/**
         * Drives the transaction process.
         */
        void startTransaction() {
			ueMiddleware = new UEMiddleware("MP200", parent, this, this);

			// Set Terminal Config
			TerminalConfig tc = TerminalConfig.getTipAdjustConfig();
			//TerminalConfig tc = TerminalConfig.getAllCVMConfig();
			//tc.setEnable_debit_msr(true);

			ueMiddleware.setTerminalConfig(tc);

			try {
				// Connect to device.
				ueMiddleware.connect(mConnType, sourceKey, pin, host);
			} catch (Exception e) {
				e.printStackTrace();
				onLogTrace(e.getLocalizedMessage());
			}
        }

		/**
		 * Explicitly disconnects the device from phone/tablet.
		 */
		public void disconnect() {
			ueMiddleware.disconnect();
		}

		/**
		 * Reacts to 'Start Transaction' button click.
		 */
		private class StartTransBtnListener implements Button.OnClickListener {
            @Override
            public void onClick(View v) {


                ueMiddleware.startTransaction(transInfo, sourceKey, pin, false, false);
                runOnUiThread(cancelTransBtnRunnable);
                btn_cancel_trans.setOnClickListener(new CancelTransBtnListener());
            }
        }

		/**
		 * Reacts to 'Cancel Transaction' button click.
		 */
        private class CancelTransBtnListener implements Button.OnClickListener {
            @Override
            public void onClick(View view) {
                ueMiddleware.cancelTransaction();
            }
        }


		/**
		 * ====================================================================
		 * ======================== Interface Methods =========================
		 * ====================================================================
		 */

		@Override
		public void onConnected() {
			Log.d(TAG, "onConnected()");
			runOnUiThread(connStatusRunnable);
			runOnUiThread(startTransBtnRunnable);
			btn_start_trans.setOnClickListener(new StartTransBtnListener());
		}

		@Override
		public void onDeviceInfoReceived(HashMap hashMap) {
			Log.d(TAG, "onDeviceInfoReceived()");
		}

		@Override
		public void onSeePhoneNFC(HashMap hashMap) {
			Log.d(TAG, "onSeePhoneNFC()");
		}

		@Override
        public void onDisconnected() {
			Log.d(TAG, "onDisconnected()");
        }

        @Override
        public void onTransactionComplete(UEMTransactionResult transResults) {
            Log.d(TAG, "onTransactionComplete()");

            Intent intent = new Intent(ProcessingActivity.this, TransFinishedActivity.class);

            runOnUiThread(tranStatusRunnable);

            // Disconnect from device.
            ueMiddleware.disconnect();

            intent.putExtra("TransResults", transResults);
            startActivity(intent);
        }

        @Override
        public void onError(UE_ERROR error) {
            Log.d(TAG, "onError() " + error);
        }

        @Override
        public void onStatusChanged(String message) {
			Log.d(TAG, "onStatusChanged(): " + message);
            mLog = "Status Changed - " + message;
            runOnUiThread(traceRunnable);
        }

        @Override
        public void onPromptForPartialAuth(String amountAuthorized, HashMap tranResults) {
			Log.d(TAG, "onPromptForPartialAuth() transaction partially approved for "+ amountAuthorized + " RefNum: " + tranResults.get("UMrefNum"));

			// Tell the middleware you want to keep the transaction (onTransactionComplete will then be called)
			ueMiddleware.returnPartialAuthDecision(true);

			// Tell the middleware that you do not want to keep the partially approved transaction.   It will be
			// voided and the funds returned to the customer
			//ueMiddleware.returnPartialAuthDecision(false);

        }

        @Override
        public void onProgressBarUpdateAvailable(String title, String message, float percent) {
			Log.d(TAG, "onProgressBarUpdateAvailable()");
        }

        @Override
        public void onReceiptReceived(String receipt) {
			Log.d(TAG, "onReceiptReceived()");
        }

		@Override
		public void onMerchantCapabilitiesReceived(JSONObject jsonObject) {
			Log.d(TAG, "onMerchantCapabilitiesReceived()");
		}

		@Override
		public void onGatewayInfoReceived(JSONObject jsonObject) {
			Log.d(TAG, "onGatewayInfoReceived()");
		}

		@Override
        public void onLog(String msg) {
			Log.d(TAG, "onLog(): " + msg);
            mLog = "Log - " + msg;
            runOnUiThread(traceRunnable);
        }

        @Override
        public void onLogTrace(String msg) {
			Log.d(TAG, "onLogTrace(): " + msg);
            mLog = "Trace - " + msg;
            runOnUiThread(traceRunnable);
        }

		/**
		 * ====================================================================
		 * ====================== Interface Methods End =======================
		 * ====================================================================
		 */


        /**
         * Displays transaction logs.
         */
        Runnable traceRunnable = new Runnable() {
            @Override
            public void run() {
                String text = tv_proc_log.getText().toString();
                text += "\n" + mLog;
                tv_proc_log.setText(text);

                sv_log.post(new Runnable() {
                    @Override
                    public void run() {
                        sv_log.fullScroll(View.FOCUS_DOWN);
                    }
                });
            }
        };

        /**
         * Handles the progress circle and transaction status TextView.
         */
        Runnable tranStatusRunnable = new Runnable() {
            @Override
            public void run() {
                tv_status.setText(getResources().getText(R.string.trans_complete));
                tv_status.setTextColor(getResources().getColor(R.color.colorWhite));
                tv_trans_status.setVisibility(View.INVISIBLE);
            }
        };

        /**
         * Lets the user know when connection is established (green 'Connected').
         */
        Runnable connStatusRunnable = new Runnable() {
            @Override
            public void run() {
                findViewById(R.id.loading_circle).setVisibility(View.GONE);
                tv_status.setText(getResources().getText(R.string.connected));
                tv_status.setTextColor(getResources().getColor(R.color.colorGreen));
                tv_trans_status.setText(getResources().getText(R.string.trans_status));
            }
        };

        /**
         * Enables 'Start Transaction' button once connection is established.
         */
        Runnable startTransBtnRunnable = new Runnable() {
            @Override
            public void run() {
                btn_start_trans.setEnabled(true);
                btn_start_trans.setBackground(getResources().getDrawable(R.drawable.rounded_button));
                btn_start_trans.setTextColor(getResources().getColor(R.color.colorWhite));
            }
        };

        /**
         * Enables 'Cancel Transaction' button once transaction has started.
         */
        Runnable cancelTransBtnRunnable = new Runnable() {
            @Override
            public void run() {
                btn_cancel_trans.setEnabled(true);
                btn_cancel_trans.setBackground(getResources().getDrawable(R.drawable.rounded_button_red));
                btn_cancel_trans.setTextColor(getResources().getColor(R.color.colorWhite));
            }
        };
    }

}
